@@ -21,7 +21,8 @@
 
 import io
 import sys
-import os.path
+import os
+import pathlib
 import subprocess
 import contextlib
 import logging
@@ -40,6 +41,11 @@ from qutebrowser.utils import version, usertypes, utils, standarddir
 from qutebrowser.misc import pastebin, objects, elf
 from qutebrowser.browser import pdfjs
 
+try:
+    from qutebrowser.browser.webengine import webenginesettings
+except ImportError:
+    webenginesettings = None
+
 
 @pytest.mark.parametrize('os_release, expected', [
     # No file
@@ -47,7 +53,7 @@ from qutebrowser.browser import pdfjs
     # Invalid file
     ("\n# foo\n foo=bar=baz",
      version.DistributionInfo(id=None, parsed=version.Distribution.unknown,
-                              version=None, pretty='Unknown')),
+                              pretty='Unknown')),
     # Archlinux
     ("""
         NAME="Arch Linux"
@@ -60,8 +66,7 @@ from qutebrowser.browser import pdfjs
         BUG_REPORT_URL="https://bugs.archlinux.org/"
      """,
      version.DistributionInfo(
-         id='arch', parsed=version.Distribution.arch, version=None,
-         pretty='Arch Linux')),
+         id='arch', parsed=version.Distribution.arch, pretty='Arch Linux')),
     # Ubuntu 14.04
     ("""
         NAME="Ubuntu"
@@ -72,9 +77,7 @@ from qutebrowser.browser import pdfjs
         VERSION_ID="14.04"
      """,
      version.DistributionInfo(
-         id='ubuntu', parsed=version.Distribution.ubuntu,
-         version=utils.VersionNumber(14, 4, 5),
-         pretty='Ubuntu 14.04.5 LTS')),
+         id='ubuntu', parsed=version.Distribution.ubuntu, pretty='Ubuntu 14.04.5 LTS')),
     # Ubuntu 17.04
     ("""
         NAME="Ubuntu"
@@ -85,9 +88,7 @@ from qutebrowser.browser import pdfjs
         VERSION_ID="17.04"
      """,
      version.DistributionInfo(
-         id='ubuntu', parsed=version.Distribution.ubuntu,
-         version=utils.VersionNumber(17, 4),
-         pretty='Ubuntu 17.04')),
+         id='ubuntu', parsed=version.Distribution.ubuntu, pretty='Ubuntu 17.04')),
     # Debian Jessie
     ("""
         PRETTY_NAME="Debian GNU/Linux 8 (jessie)"
@@ -98,7 +99,6 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='debian', parsed=version.Distribution.debian,
-         version=utils.VersionNumber(8),
          pretty='Debian GNU/Linux 8 (jessie)')),
     # Void Linux
     ("""
@@ -108,8 +108,7 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="void"
      """,
      version.DistributionInfo(
-         id='void', parsed=version.Distribution.void,
-         version=None, pretty='void')),
+         id='void', parsed=version.Distribution.void, pretty='void')),
     # Gentoo
     ("""
         NAME=Gentoo
@@ -117,8 +116,7 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="Gentoo/Linux"
      """,
      version.DistributionInfo(
-         id='gentoo', parsed=version.Distribution.gentoo,
-         version=None, pretty='Gentoo/Linux')),
+         id='gentoo', parsed=version.Distribution.gentoo, pretty='Gentoo/Linux')),
     # Fedora
     ("""
         NAME=Fedora
@@ -129,7 +127,6 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='fedora', parsed=version.Distribution.fedora,
-         version=utils.VersionNumber(25),
          pretty='Fedora 25 (Twenty Five)')),
     # OpenSUSE
     ("""
@@ -142,7 +139,6 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='opensuse', parsed=version.Distribution.opensuse,
-         version=utils.VersionNumber(42, 2),
          pretty='openSUSE Leap 42.2')),
     # Linux Mint
     ("""
@@ -155,7 +151,6 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='linuxmint', parsed=version.Distribution.linuxmint,
-         version=utils.VersionNumber(18, 1),
          pretty='Linux Mint 18.1')),
     # Manjaro
     ("""
@@ -164,8 +159,7 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="Manjaro Linux"
      """,
      version.DistributionInfo(
-         id='manjaro', parsed=version.Distribution.manjaro,
-         version=None, pretty='Manjaro Linux')),
+         id='manjaro', parsed=version.Distribution.manjaro, pretty='Manjaro Linux')),
     # Funtoo
     ("""
         ID="funtoo"
@@ -173,8 +167,7 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="Linux"
      """,
      version.DistributionInfo(
-         id='funtoo', parsed=version.Distribution.gentoo,
-         version=None, pretty='Funtoo GNU/Linux')),
+         id='funtoo', parsed=version.Distribution.gentoo, pretty='Funtoo GNU/Linux')),
     # KDE neon
     ("""
         NAME="KDE neon"
@@ -187,7 +180,7 @@ from qutebrowser.browser import pdfjs
     """,
     version.DistributionInfo(
         id='neon', parsed=version.Distribution.neon,
-        version=utils.VersionNumber(5, 20), pretty='KDE neon User Edition 5.20')),
+        pretty='KDE neon User Edition 5.20')),
     # Archlinux ARM
     ("""
         NAME="Arch Linux ARM"
@@ -196,8 +189,7 @@ from qutebrowser.browser import pdfjs
         ID_LIKE=arch
     """,
     version.DistributionInfo(
-        id='archarm', parsed=version.Distribution.arch,
-        version=None, pretty='Arch Linux ARM')),
+        id='archarm', parsed=version.Distribution.arch, pretty='Arch Linux ARM')),
     # Alpine
     ("""
         NAME="Alpine Linux"
@@ -206,8 +198,7 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="Alpine Linux edge"
     """,
     version.DistributionInfo(
-        id='alpine', parsed=version.Distribution.alpine,
-        version=utils.VersionNumber(3, 12), pretty='Alpine Linux edge')),
+        id='alpine', parsed=version.Distribution.alpine, pretty='Alpine Linux edge')),
     # EndeavourOS
     ("""
         NAME="EndeavourOS"
@@ -219,8 +210,7 @@ from qutebrowser.browser import pdfjs
         LOGO=endeavouros
     """,
     version.DistributionInfo(
-        id='endeavouros', parsed=version.Distribution.arch,
-        version=None, pretty='EndeavourOS')),
+        id='endeavouros', parsed=version.Distribution.arch, pretty='EndeavourOS')),
     # Manjaro ARM
     ("""
         NAME="Manjaro-ARM"
@@ -229,8 +219,7 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="Manjaro ARM"
     """,
     version.DistributionInfo(
-        id='manjaro-arm', parsed=version.Distribution.manjaro,
-        version=None, pretty='Manjaro ARM')),
+        id='manjaro-arm', parsed=version.Distribution.manjaro, pretty='Manjaro ARM')),
     # Artix Linux
     ("""
         NAME="Artix Linux"
@@ -238,8 +227,7 @@ from qutebrowser.browser import pdfjs
         ID=artix
     """,
     version.DistributionInfo(
-        id='artix', parsed=version.Distribution.arch,
-        version=None, pretty='Artix Linux')),
+        id='artix', parsed=version.Distribution.arch, pretty='Artix Linux')),
     # NixOS
     ("""
         NAME=NixOS
@@ -250,9 +238,15 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="NixOS 21.03 (Okapi)"
     """,
     version.DistributionInfo(
-        id='nixos', parsed=version.Distribution.nixos,
-        version=utils.VersionNumber(21, 3),
-        pretty='NixOS 21.03 (Okapi)')),
+        id='nixos', parsed=version.Distribution.nixos, pretty='NixOS 21.03 (Okapi)')),
+    # NixOS (fake fourth version component)
+    ("""
+        NAME=NixOS
+        ID=nixos
+        VERSION="21.05.20210402.1dead (Okapi)"
+    """,
+    version.DistributionInfo(
+        id='nixos', parsed=version.Distribution.nixos, pretty='NixOS')),
     # SolusOS
     ("""
         NAME="Solus"
@@ -263,9 +257,7 @@ from qutebrowser.browser import pdfjs
         PRETTY_NAME="Solus 4.2 Fortitude"
     """,
     version.DistributionInfo(
-        id='solus', parsed=version.Distribution.solus,
-        version=utils.VersionNumber(4, 2),
-        pretty='Solus 4.2 Fortitude')),
+        id='solus', parsed=version.Distribution.solus, pretty='Solus 4.2 Fortitude')),
     # KDE Platform
     ("""
         NAME=KDE
@@ -274,17 +266,14 @@ from qutebrowser.browser import pdfjs
         ID=org.kde.Platform
     """,
     version.DistributionInfo(
-        id='org.kde.Platform', parsed=version.Distribution.kde_flatpak,
-        version=utils.VersionNumber(5, 12),
-        pretty='KDE')),
+        id='org.kde.Platform', parsed=version.Distribution.kde_flatpak, pretty='KDE')),
     # No PRETTY_NAME
     ("""
         NAME="Tux"
         ID=tux
     """,
     version.DistributionInfo(
-        id='tux', parsed=version.Distribution.unknown,
-        version=None, pretty='Tux')),
+        id='tux', parsed=version.Distribution.unknown, pretty='Tux')),
     # Invalid multi-line value
     ("""
         ID=tux
@@ -292,13 +281,12 @@ from qutebrowser.browser import pdfjs
         Text"
     """,
     version.DistributionInfo(
-        id='tux', parsed=version.Distribution.unknown,
-        version=None, pretty='Multiline')),
+        id='tux', parsed=version.Distribution.unknown, pretty='Multiline')),
 ])
-def test_distribution(tmpdir, monkeypatch, os_release, expected):
-    os_release_file = tmpdir / 'os-release'
+def test_distribution(tmp_path, monkeypatch, os_release, expected):
+    os_release_file = tmp_path / 'os-release'
     if os_release is not None:
-        os_release_file.write(textwrap.dedent(os_release))
+        os_release_file.write_text(textwrap.dedent(os_release), encoding="utf-8")
     monkeypatch.setenv('QUTE_FAKE_OS_RELEASE', str(os_release_file))
 
     assert version.distribution() == expected
@@ -352,9 +340,8 @@ class GitStrSubprocessFake:
             raise ValueError("func got called without retval being set!")
         retval = self.retval
         self.retval = self.UNSET
-        gitpath = os.path.normpath(gitpath)
-        expected = os.path.abspath(os.path.join(
-            os.path.dirname(qutebrowser.__file__), os.pardir))
+        gitpath = pathlib.Path(gitpath).resolve()
+        expected = pathlib.Path(qutebrowser.__file__).parent.parent
         assert gitpath == expected
         return retval
 
@@ -457,7 +444,7 @@ class TestGitStrSubprocess:
     """Tests for _git_str_subprocess."""
 
     @pytest.fixture
-    def git_repo(self, tmpdir):
+    def git_repo(self, tmp_path):
         """A fixture to create a temporary git repo.
 
         Some things are tested against a real repo so we notice if something in
@@ -479,37 +466,46 @@ class TestGitStrSubprocess:
                 # some environments on Windows...
                 # https://bugs.python.org/issue24493
                 subprocess.run(
-                    'git -C "{}" {}'.format(tmpdir, ' '.join(args)),
+                    'git -C "{}" {}'.format(tmp_path, ' '.join(args)),
                     env=env, check=True, shell=True)
             else:
                 subprocess.run(
-                    ['git', '-C', str(tmpdir)] + list(args),
+                    ['git', '-C', str(tmp_path)] + list(args),
                     check=True, env=env)
 
-        (tmpdir / 'file').write_text("Hello World!", encoding='utf-8')
+        (tmp_path / 'file').write_text("Hello World!", encoding='utf-8')
         _git('init')
         _git('add', 'file')
         _git('commit', '-am', 'foo', '--no-verify', '--no-edit',
              '--no-post-rewrite', '--quiet', '--no-gpg-sign')
         _git('tag', 'foobar')
-        return tmpdir
+        return tmp_path
 
     @needs_git
     def test_real_git(self, git_repo):
         """Test with a real git repository."""
-        ret = version._git_str_subprocess(str(git_repo))
-        assert ret == '6e4b65a on master (1970-01-01 01:00:00 +0100)'
+        branch_name = subprocess.run(
+            ['git', 'config', 'init.defaultBranch'],
+            check=False,
+            stdout=subprocess.PIPE,
+            encoding='utf-8',
+        ).stdout.strip()
+        if not branch_name:
+            branch_name = 'master'
 
-    def test_missing_dir(self, tmpdir):
+        ret = version._git_str_subprocess(str(git_repo))
+        assert ret == f'6e4b65a on {branch_name} (1970-01-01 01:00:00 +0100)'
+
+    def test_missing_dir(self, tmp_path):
         """Test with a directory which doesn't exist."""
-        ret = version._git_str_subprocess(str(tmpdir / 'does-not-exist'))
+        ret = version._git_str_subprocess(str(tmp_path / 'does-not-exist'))
         assert ret is None
 
     @pytest.mark.parametrize('exc', [
         OSError,
         subprocess.CalledProcessError(1, 'foobar')
     ])
-    def test_exception(self, exc, mocker, tmpdir):
+    def test_exception(self, exc, mocker, tmp_path):
         """Test with subprocess.run raising an exception.
 
         Args:
@@ -519,7 +515,7 @@ class TestGitStrSubprocess:
         m.path.isdir.return_value = True
         mocker.patch('qutebrowser.utils.version.subprocess.run',
                      side_effect=exc)
-        ret = version._git_str_subprocess(str(tmpdir))
+        ret = version._git_str_subprocess(str(tmp_path))
         assert ret is None
 
 
@@ -978,7 +974,7 @@ class TestWebEngineVersions:
         )
         assert version.WebEngineVersions.from_elf(elf_version) == expected
 
-    @pytest.mark.parametrize('qt_version, chromium_version', [
+    @pytest.mark.parametrize('pyqt_version, chromium_version', [
         ('5.12.10', '69.0.3497.128'),
         ('5.14.2', '77.0.3865.129'),
         ('5.15.1', '80.0.3987.163'),
@@ -987,13 +983,19 @@ class TestWebEngineVersions:
         ('5.15.4', '87.0.4280.144'),
         ('5.15.5', '87.0.4280.144'),
     ])
-    def test_from_pyqt(self, qt_version, chromium_version):
+    def test_from_pyqt(self, freezer, pyqt_version, chromium_version):
+        if freezer and pyqt_version in ['5.15.3', '5.15.4', '5.15.5']:
+            chromium_version = '83.0.4103.122'
+            expected_pyqt_version = '5.15.2'
+        else:
+            expected_pyqt_version = pyqt_version
+
         expected = version.WebEngineVersions(
-            webengine=utils.VersionNumber.parse(qt_version),
+            webengine=utils.VersionNumber.parse(expected_pyqt_version),
             chromium=chromium_version,
             source='PyQt',
         )
-        assert version.WebEngineVersions.from_pyqt(qt_version) == expected
+        assert version.WebEngineVersions.from_pyqt(pyqt_version) == expected
 
     def test_real_chromium_version(self, qapp):
         """Compare the inferred Chromium version with the real one."""
@@ -1018,7 +1020,6 @@ class TestWebEngineVersions:
         versions = version.WebEngineVersions.from_pyqt(pyqt_webengine_version)
         inferred = versions.chromium
 
-        from qutebrowser.browser.webengine import webenginesettings
         webenginesettings.init_user_agent()
         real = webenginesettings.parsed_user_agent.upstream_browser_version
 
@@ -1059,26 +1060,24 @@ class TestChromiumVersion:
     @pytest.fixture(autouse=True)
     def clear_parsed_ua(self, monkeypatch):
         pytest.importorskip('PyQt5.QtWebEngineWidgets')
-        if version.webenginesettings is not None:
+        if webenginesettings is not None:
             # Not available with QtWebKit
-            monkeypatch.setattr(version.webenginesettings, 'parsed_user_agent', None)
+            monkeypatch.setattr(webenginesettings, 'parsed_user_agent', None)
 
     def test_fake_ua(self, monkeypatch, caplog):
         ver = '77.0.3865.98'
-        version.webenginesettings._init_user_agent_str(
-            _QTWE_USER_AGENT.format(ver))
+        webenginesettings._init_user_agent_str(_QTWE_USER_AGENT.format(ver))
 
         assert version.qtwebengine_versions().chromium == ver
 
     def test_prefers_saved_user_agent(self, monkeypatch):
-        version.webenginesettings._init_user_agent_str(_QTWE_USER_AGENT.format('87'))
+        webenginesettings._init_user_agent_str(_QTWE_USER_AGENT.format('87'))
 
         class FakeProfile:
             def defaultProfile(self):
                 raise AssertionError("Should not be called")
 
-        monkeypatch.setattr(version.webenginesettings, 'QWebEngineProfile',
-                            FakeProfile())
+        monkeypatch.setattr(webenginesettings, 'QWebEngineProfile', FakeProfile())
 
         version.qtwebengine_versions()
 
@@ -1175,6 +1174,16 @@ class TestChromiumVersion:
         assert versions.source == 'importlib'
         assert versions.webengine == expected
 
+    @pytest.mark.parametrize('override', [
+        utils.VersionNumber(5, 12, 10),
+        utils.VersionNumber(5, 15, 3),
+    ])
+    def test_override(self, monkeypatch, override):
+        monkeypatch.setenv('QUTE_QTWEBENGINE_VERSION_OVERRIDE', str(override))
+        versions = version.qtwebengine_versions(avoid_init=True)
+        assert versions.source == 'override'
+        assert versions.webengine == override
+
 
 @dataclasses.dataclass
 class VersionParams:
@@ -1204,10 +1213,10 @@ class VersionParams:
 def test_version_info(params, stubs, monkeypatch, config_stub):
     """Test version.version_info()."""
     config.instance.config_py_loaded = params.config_py_loaded
-    import_path = os.path.abspath('/IMPORTPATH')
+    import_path = pathlib.Path('/IMPORTPATH').resolve()
 
     patches = {
-        'qutebrowser.__file__': os.path.join(import_path, '__init__.py'),
+        'qutebrowser.__file__': str(import_path / '__init__.py'),
         'qutebrowser.__version__': 'VERSION',
         '_git_str': lambda: ('GIT COMMIT' if params.git_commit else None),
         'platform.python_implementation': lambda: 'PYTHON IMPLEMENTATION',
@@ -1264,7 +1273,6 @@ def test_version_info(params, stubs, monkeypatch, config_stub):
     if params.with_webkit:
         patches['qWebKitVersion'] = lambda: 'WEBKIT VERSION'
         patches['objects.backend'] = usertypes.Backend.QtWebKit
-        patches['webenginesettings'] = None
         substitutions['backend'] = 'new QtWebKit (WebKit WEBKIT VERSION)'
     else:
         monkeypatch.delattr(version, 'qtutils.qWebKitVersion', raising=False)
@@ -1273,8 +1281,7 @@ def test_version_info(params, stubs, monkeypatch, config_stub):
 
     if params.known_distribution:
         patches['distribution'] = lambda: version.DistributionInfo(
-            parsed=version.Distribution.arch, version=None,
-            pretty='LINUX DISTRIBUTION', id='arch')
+            parsed=version.Distribution.arch, pretty='LINUX DISTRIBUTION', id='arch')
         substitutions['linuxdist'] = ('\nLinux distribution: '
                                       'LINUX DISTRIBUTION (arch)')
         substitutions['osinfo'] = ''
